@@ -35,24 +35,24 @@ function check_upload_size( $file ) {
 	if ( upload_is_user_over_quota( false ) ) {
 		$file['error'] = __( 'You have used your space quota. Please delete files before uploading.' );
 	}
-	if ( $file['error'] != '0' && !isset($_POST['html-upload']) )
+	if ( $file['error'] != '0' && ! isset( $_POST['html-upload'] ) && ( ! defined( 'DOING_AJAX' ) || ! DOING_AJAX ) ) {
 		wp_die( $file['error'] . ' <a href="javascript:history.go(-1)">' . __( 'Back' ) . '</a>' );
+	}
 
 	return $file;
 }
 add_filter( 'wp_handle_upload_prefilter', 'check_upload_size' );
 
 /**
- * Delete a blog
+ * Delete a blog.
  *
  * @since 3.0.0
  *
- * @param int $blog_id Blog ID
- * @param bool $drop True if blog's table should be dropped. Default is false.
- * @return void
+ * @param int  $blog_id Blog ID.
+ * @param bool $drop    True if blog's table should be dropped. Default is false.
  */
 function wpmu_delete_blog( $blog_id, $drop = false ) {
-	global $wpdb, $current_site;
+	global $wpdb;
 
 	$switch = false;
 	if ( get_current_blog_id() != $blog_id ) {
@@ -61,7 +61,14 @@ function wpmu_delete_blog( $blog_id, $drop = false ) {
 	}
 
 	$blog = get_blog_details( $blog_id );
-
+	/**
+	 * Fires before a blog is deleted.
+	 *
+	 * @since MU
+	 *
+	 * @param int  $blog_id The blog ID.
+	 * @param bool $drop    True if blog's table should be dropped. Default is false.
+	 */
 	do_action( 'delete_blog', $blog_id, $drop );
 
 	$users = get_users( array( 'blog_id' => $blog_id, 'fields' => 'ids' ) );
@@ -75,12 +82,38 @@ function wpmu_delete_blog( $blog_id, $drop = false ) {
 
 	update_blog_status( $blog_id, 'deleted', 1 );
 
-	// Don't destroy the initial, main, or root blog.
-	if ( $drop && ( 1 == $blog_id || is_main_site( $blog_id ) || ( $blog->path == $current_site->path && $blog->domain == $current_site->domain ) ) )
+	$current_site = get_current_site();
+
+	// If a full blog object is not available, do not destroy anything.
+	if ( $drop && ! $blog ) {
 		$drop = false;
+	}
+
+	// Don't destroy the initial, main, or root blog.
+	if ( $drop && ( 1 == $blog_id || is_main_site( $blog_id ) || ( $blog->path == $current_site->path && $blog->domain == $current_site->domain ) ) ) {
+		$drop = false;
+	}
+
+	$upload_path = trim( get_option( 'upload_path' ) );
+
+	// If ms_files_rewriting is enabled and upload_path is empty, wp_upload_dir is not reliable.
+	if ( $drop && get_site_option( 'ms_files_rewriting' ) && empty( $upload_path ) ) {
+		$drop = false;
+	}
 
 	if ( $drop ) {
-		$drop_tables = apply_filters( 'wpmu_drop_tables', $wpdb->tables( 'blog' ), $blog_id );
+		$uploads = wp_upload_dir();
+
+		$tables = $wpdb->tables( 'blog' );
+		/**
+		 * Filter the tables to drop when the blog is deleted.
+		 *
+		 * @since MU
+		 *
+		 * @param array $tables  The blog tables to be dropped.
+		 * @param int   $blog_id The ID of the blog to drop tables for.
+		 */
+		$drop_tables = apply_filters( 'wpmu_drop_tables', $tables, $blog_id );
 
 		foreach ( (array) $drop_tables as $table ) {
 			$wpdb->query( "DROP TABLE IF EXISTS `$table`" );
@@ -88,7 +121,14 @@ function wpmu_delete_blog( $blog_id, $drop = false ) {
 
 		$wpdb->delete( $wpdb->blogs, array( 'blog_id' => $blog_id ) );
 
-		$uploads = wp_upload_dir();
+		/**
+		 * Filter the upload base directory to delete when the blog is deleted.
+		 *
+		 * @since MU
+		 *
+		 * @param string $uploads['basedir'] Uploads path without subdirectory. @see wp_upload_dir()
+		 * @param int    $blog_id            The blog ID.
+		 */
 		$dir = apply_filters( 'wpmu_delete_blog_upload_dir', $uploads['basedir'], $blog_id );
 		$dir = rtrim( $dir, DIRECTORY_SEPARATOR );
 		$top_dir = $dir;
@@ -128,7 +168,16 @@ function wpmu_delete_blog( $blog_id, $drop = false ) {
 		restore_current_blog();
 }
 
-// @todo Merge with wp_delete_user() ?
+/**
+ * Delete a user from the network and remove from all sites.
+ *
+ * @since 3.0.0
+ *
+ * @todo Merge with wp_delete_user() ?
+ *
+ * @param int $id The user ID.
+ * @return bool True if the user was deleted, otherwise false.
+ */
 function wpmu_delete_user( $id ) {
 	global $wpdb;
 
@@ -137,7 +186,13 @@ function wpmu_delete_user( $id ) {
 
 	if ( !$user->exists() )
 		return false;
-
+	/**
+	 * Fires before a user is deleted from the network.
+	 *
+	 * @since MU
+	 *
+	 * @param int $id ID of the user about to be deleted from the network.
+	 */
 	do_action( 'wpmu_delete_user', $id );
 
 	$blogs = get_blogs_of_user( $id );
@@ -172,14 +227,21 @@ function wpmu_delete_user( $id ) {
 
 	clean_user_cache( $user );
 
-	// allow for commit transaction
+	/** This action is documented in wp-admin/includes/user.php */
 	do_action( 'deleted_user', $id );
 
 	return true;
 }
 
+/**
+ * Sends an email when a site administrator email address is changed.
+ *
+ * @since 3.0.0
+ *
+ * @param string $old_value The old email address. Not currently used.
+ * @param string $value     The new email address.
+ */
 function update_option_new_admin_email( $old_value, $value ) {
-	$email = get_option( 'admin_email' );
 	if ( $value == get_option( 'admin_email' ) || !is_email( $value ) )
 		return;
 
@@ -190,7 +252,7 @@ function update_option_new_admin_email( $old_value, $value ) {
 	);
 	update_option( 'adminhash', $new_admin_email );
 
-	$content = apply_filters( 'new_admin_email_content', __( "Dear user,
+	$email_text = __( 'Dear user,
 
 You recently requested to have the administration email address on
 your site changed.
@@ -204,18 +266,42 @@ This email has been sent to ###EMAIL###
 
 Regards,
 All at ###SITENAME###
-###SITEURL### "), $new_admin_email );
+###SITEURL###' );
+
+	/**
+	 * Filter the email text sent when the site admin email is changed.
+	 *
+	 * The following strings have a special meaning and will get replaced dynamically:
+	 * ###ADMIN_URL### The link to click on to confirm the email change. Required otherwise this functunalty is will break.
+	 * ###EMAIL###     The new email.
+	 * ###SITENAME###  The name of the site.
+	 * ###SITEURL###   The URL to the site.
+	 *
+	 * @since MU
+	 *
+	 * @param string $email_text      Text in the email.
+	 * @param string $new_admin_email New admin email that the current administration email was changed to.
+	 */
+	$content = apply_filters( 'new_admin_email_content', $email_text, $new_admin_email );
 
 	$content = str_replace( '###ADMIN_URL###', esc_url( admin_url( 'options.php?adminhash='.$hash ) ), $content );
 	$content = str_replace( '###EMAIL###', $value, $content );
 	$content = str_replace( '###SITENAME###', get_site_option( 'site_name' ), $content );
 	$content = str_replace( '###SITEURL###', network_home_url(), $content );
 
-	wp_mail( $value, sprintf( __( '[%s] New Admin Email Address' ), get_option( 'blogname' ) ), $content );
+	wp_mail( $value, sprintf( __( '[%s] New Admin Email Address' ), wp_specialchars_decode( get_option( 'blogname' ) ) ), $content );
 }
 add_action( 'update_option_new_admin_email', 'update_option_new_admin_email', 10, 2 );
 add_action( 'add_option_new_admin_email', 'update_option_new_admin_email', 10, 2 );
 
+/**
+ * Sends an email when an email address change is requested.
+ *
+ * @since 3.0.0
+ *
+ * @global object $errors WP_Error object.
+ * @global object $wpdb   WordPress database object.
+ */
 function send_confirmation_on_profile_email() {
 	global $errors, $wpdb;
 	$current_user = wp_get_current_user();
@@ -244,7 +330,7 @@ function send_confirmation_on_profile_email() {
 				);
 		update_option( $current_user->ID . '_new_email', $new_user_email );
 
-		$content = apply_filters( 'new_user_email_content', __( "Dear user,
+		$email_text = __( 'Dear user,
 
 You recently requested to have the email address on your account changed.
 If this is correct, please click on the following link to change it:
@@ -257,19 +343,41 @@ This email has been sent to ###EMAIL###
 
 Regards,
 All at ###SITENAME###
-###SITEURL###" ), $new_user_email );
+###SITEURL###' );
+
+		/**
+		 * Filter the email text sent when a user changes emails.
+		 *
+		 * The following strings have a special meaning and will get replaced dynamically:
+		 * ###ADMIN_URL### The link to click on to confirm the email change. Required otherwise this functunalty is will break.
+		 * ###EMAIL### The new email.
+		 * ###SITENAME### The name of the site.
+		 * ###SITEURL### The URL to the site.
+		 *
+		 * @since MU
+		 *
+		 * @param string $email_text     Text in the email.
+		 * @param string $new_user_email New user email that the current user has changed to.
+		 */
+		$content = apply_filters( 'new_user_email_content', $email_text, $new_user_email );
 
 		$content = str_replace( '###ADMIN_URL###', esc_url( admin_url( 'profile.php?newuseremail='.$hash ) ), $content );
 		$content = str_replace( '###EMAIL###', $_POST['email'], $content);
 		$content = str_replace( '###SITENAME###', get_site_option( 'site_name' ), $content );
 		$content = str_replace( '###SITEURL###', network_home_url(), $content );
 
-		wp_mail( $_POST['email'], sprintf( __( '[%s] New Email Address' ), get_option( 'blogname' ) ), $content );
+		wp_mail( $_POST['email'], sprintf( __( '[%s] New Email Address' ), wp_specialchars_decode( get_option( 'blogname' ) ) ), $content );
 		$_POST['email'] = $current_user->user_email;
 	}
 }
 add_action( 'personal_options_update', 'send_confirmation_on_profile_email' );
 
+/**
+ * Adds an admin notice alerting the user to check for confirmation email
+ * after email address change.
+ *
+ * @since 3.0.0
+ */
 function new_user_email_admin_notice() {
 	if ( strpos( $_SERVER['PHP_SELF'], 'profile.php' ) && isset( $_GET['updated'] ) && $email = get_option( get_current_user_id() . '_new_email' ) )
 		echo "<div class='update-nag'>" . sprintf( __( "Your email address has not been updated yet. Please check your inbox at %s for a confirmation email." ), $email['newemail'] ) . "</div>";
@@ -282,7 +390,7 @@ add_action( 'admin_notices', 'new_user_email_admin_notice' );
  * @since MU
  *
  * @param bool $echo Optional. If $echo is set and the quota is exceeded, a warning message is echoed. Default is true.
- * @return int
+ * @return bool True if user is over upload space quota, otherwise false.
  */
 function upload_is_user_over_quota( $echo = true ) {
 	if ( get_site_option( 'upload_space_check_disabled' ) )
@@ -332,9 +440,6 @@ function display_space_usage() {
  * Get the remaining upload space for this blog.
  *
  * @since MU
- * @uses upload_is_user_over_quota()
- * @uses get_space_allowed()
- * @uses get_upload_space_available()
  *
  * @param int $size Current max size in bytes
  * @return int Max size in bytes
@@ -347,7 +452,13 @@ function fix_import_form_size( $size ) {
 	return min( $size, $available );
 }
 
-// Edit blog upload space setting on Edit Blog page
+/**
+ * Displays the edit blog upload space setting form on the Edit Blog screen.
+ *
+ * @since 3.0.0
+ *
+ * @param int $id The ID of the blog to display the setting for.
+ */
 function upload_space_setting( $id ) {
 	switch_to_blog( $id );
 	$quota = get_option( 'blog_upload_space' );
@@ -365,6 +476,20 @@ function upload_space_setting( $id ) {
 }
 add_action( 'wpmueditblogaction', 'upload_space_setting' );
 
+/**
+ * Update the status of a user in the database.
+ *
+ * Used in core to mark a user as spam or "ham" (not spam) in Multisite.
+ *
+ * @since 3.0.0
+ *
+ * @param int    $id         The user ID.
+ * @param string $pref       The column in the wp_users table to update the user's status
+ *                           in (presumably user_status, spam, or deleted).
+ * @param int    $value      The new status for the user.
+ * @param null   $deprecated Deprecated as of 3.0.2 and should not be used.
+ * @return int   The initially passed $value.
+ */
 function update_user_status( $id, $pref, $value, $deprecated = null ) {
 	global $wpdb;
 
@@ -377,15 +502,38 @@ function update_user_status( $id, $pref, $value, $deprecated = null ) {
 	clean_user_cache( $user );
 
 	if ( $pref == 'spam' ) {
-		if ( $value == 1 )
+		if ( $value == 1 ) {
+			/**
+			 * Fires after the user is marked as a SPAM user.
+			 *
+			 * @since 3.0.0
+			 *
+			 * @param int $id ID of the user marked as SPAM.
+			 */
 			do_action( 'make_spam_user', $id );
-		else
+		} else {
+			/**
+			 * Fires after the user is marked as a HAM user. Opposite of SPAM.
+			 *
+			 * @since 3.0.0
+			 *
+			 * @param int $id ID of the user marked as HAM.
+			 */
 			do_action( 'make_ham_user', $id );
+		}
 	}
 
 	return $value;
 }
 
+/**
+ * Cleans the user cache for a specific user.
+ *
+ * @since 3.0.0
+ *
+ * @param int $id The user ID.
+ * @return bool|int The ID of the refreshed user or false if the user does not exist.
+ */
 function refresh_user_details( $id ) {
 	$id = (int) $id;
 
@@ -397,6 +545,15 @@ function refresh_user_details( $id ) {
 	return $id;
 }
 
+/**
+ * Returns the language for a language code.
+ *
+ * @since 3.0.0
+ *
+ * @param string $code Optional. The two-letter language code. Default empty.
+ * @return string The language corresponding to $code if it exists. If it does not exist,
+ *                then the first two letters of $code is returned.
+ */
 function format_code_lang( $code = '' ) {
 	$code = strtolower( substr( $code, 0, 2 ) );
 	$lang_codes = array(
@@ -411,10 +568,31 @@ function format_code_lang( $code = '' ) {
 		'sg' => 'Sango', 'sa' => 'Sanskrit', 'sr' => 'Serbian', 'hr' => 'Croatian', 'si' => 'Sinhala; Sinhalese', 'sk' => 'Slovak', 'sl' => 'Slovenian', 'se' => 'Northern Sami', 'sm' => 'Samoan', 'sn' => 'Shona', 'sd' => 'Sindhi', 'so' => 'Somali', 'st' => 'Sotho, Southern', 'es' => 'Spanish; Castilian', 'sc' => 'Sardinian', 'ss' => 'Swati', 'su' => 'Sundanese', 'sw' => 'Swahili',
 		'sv' => 'Swedish', 'ty' => 'Tahitian', 'ta' => 'Tamil', 'tt' => 'Tatar', 'te' => 'Telugu', 'tg' => 'Tajik', 'tl' => 'Tagalog', 'th' => 'Thai', 'bo' => 'Tibetan', 'ti' => 'Tigrinya', 'to' => 'Tonga (Tonga Islands)', 'tn' => 'Tswana', 'ts' => 'Tsonga', 'tk' => 'Turkmen', 'tr' => 'Turkish', 'tw' => 'Twi', 'ug' => 'Uighur; Uyghur', 'uk' => 'Ukrainian', 'ur' => 'Urdu', 'uz' => 'Uzbek',
 		've' => 'Venda', 'vi' => 'Vietnamese', 'vo' => 'Volapük', 'cy' => 'Welsh','wa' => 'Walloon','wo' => 'Wolof', 'xh' => 'Xhosa', 'yi' => 'Yiddish', 'yo' => 'Yoruba', 'za' => 'Zhuang; Chuang', 'zu' => 'Zulu' );
+
+	/**
+	 * Filter the language codes.
+	 *
+	 * @since MU
+	 *
+	 * @param array  $lang_codes Key/value pair of language codes where key is the short version.
+	 * @param string $code       A two-letter designation of the language.
+	 */
 	$lang_codes = apply_filters( 'lang_codes', $lang_codes, $code );
 	return strtr( $code, $lang_codes );
 }
 
+/**
+ * Synchronize category and post tag slugs when global terms are enabled.
+ *
+ * @since 3.0.0
+ *
+ * @param $term     The term.
+ * @param $taxonomy The taxonomy for $term. Should be 'category' or 'post_tag', as these are
+ *                  the only taxonomies which are processed by this function; anything else
+ *                  will be returned untouched.
+ * @return object|array Returns `$term`, after filtering the 'slug' field with {@see sanitize_title()}
+ *                      if $taxonomy is 'category' or 'post_tag'.
+ */
 function sync_category_tag_slugs( $term, $taxonomy ) {
 	if ( global_terms_enabled() && ( $taxonomy == 'category' || $taxonomy == 'post_tag' ) ) {
 		if ( is_object( $term ) ) {
@@ -427,6 +605,13 @@ function sync_category_tag_slugs( $term, $taxonomy ) {
 }
 add_filter( 'get_term', 'sync_category_tag_slugs', 10, 2 );
 
+/**
+ * Displays an access denied message when a user tries to view a site's dashboard they
+ * do not have access to.
+ *
+ * @since 3.2.0
+ * @access private
+ */
 function _access_denied_splash() {
 	if ( ! is_user_logged_in() || is_network_admin() )
 		return;
@@ -448,21 +633,27 @@ function _access_denied_splash() {
 	$output .= '<table>';
 
 	foreach ( $blogs as $blog ) {
-		$output .= "<tr>";
-		$output .= "<td valign='top'>";
-		$output .= "{$blog->blogname}";
-		$output .= "</td>";
-		$output .= "<td valign='top'>";
-		$output .= "<a href='" . esc_url( get_admin_url( $blog->userblog_id ) ) . "'>" . __( 'Visit Dashboard' ) . "</a> | <a href='" . esc_url( get_home_url( $blog->userblog_id ) ). "'>" . __( 'View Site' ) . "</a>" ;
-		$output .= "</td>";
-		$output .= "</tr>";
+		$output .= '<tr>';
+		$output .= "<td>{$blog->blogname}</td>";
+		$output .= '<td><a href="' . esc_url( get_admin_url( $blog->userblog_id ) ) . '">' . __( 'Visit Dashboard' ) . '</a> | ' .
+			'<a href="' . esc_url( get_home_url( $blog->userblog_id ) ). '">' . __( 'View Site' ) . '</a></td>';
+		$output .= '</tr>';
 	}
+
 	$output .= '</table>';
 
 	wp_die( $output );
 }
 add_action( 'admin_page_access_denied', '_access_denied_splash', 99 );
 
+/**
+ * Checks if the current user has permissions to import new users.
+ *
+ * @since 3.0.0
+ *
+ * @param string $permission A permission to be checked. Currently not used.
+ * @return bool True if the user has proper permissions, false if they do not.
+ */
 function check_import_new_users( $permission ) {
 	if ( !is_super_admin() )
 		return false;
@@ -471,6 +662,14 @@ function check_import_new_users( $permission ) {
 add_filter( 'import_allow_create_users', 'check_import_new_users' );
 // See "import_allow_fetch_attachments" and "import_attachment_size_limit" filters too.
 
+/**
+ * Generates and displays a drop-down of available languages.
+ *
+ * @since 3.0.0
+ *
+ * @param array  $lang_files Optional. An array of the language files. Default empty array.
+ * @param string $current    Optional. The current language code. Default empty.
+ */
 function mu_dropdown_languages( $lang_files = array(), $current = '' ) {
 	$flag = false;
 	$output = array();
@@ -499,30 +698,27 @@ function mu_dropdown_languages( $lang_files = array(), $current = '' ) {
 	// Order by name
 	uksort( $output, 'strnatcasecmp' );
 
+	/**
+	 * Filter the languages available in the dropdown.
+	 *
+	 * @since MU
+	 *
+	 * @param array $output     HTML output of the dropdown.
+	 * @param array $lang_files Available language files.
+	 * @param string $current   The current language code.
+	 */
 	$output = apply_filters( 'mu_dropdown_languages', $output, $lang_files, $current );
+
 	echo implode( "\n\t", $output );
 }
 
-/* Warn the admin if SECRET SALT information is missing from wp-config.php */
-function secret_salt_warning() {
-	if ( !is_super_admin() )
-		return;
-	$secret_keys = array( 'AUTH_KEY', 'SECURE_AUTH_KEY', 'LOGGED_IN_KEY', 'NONCE_KEY', 'AUTH_SALT', 'SECURE_AUTH_SALT', 'LOGGED_IN_SALT', 'NONCE_SALT' );
-	$out = '';
-	foreach( $secret_keys as $key ) {
-		if ( ! defined( $key ) )
-			$out .= "define( '$key', '" . esc_html( wp_generate_password( 64, true, true ) ) . "' );<br />";
-	}
-	if ( $out != '' ) {
-		$msg  = __( 'Warning! WordPress encrypts user cookies, but you must add the following lines to <strong>wp-config.php</strong> for it to be more secure.' );
-		$msg .= '<br/>' . __( "Before the line <code>/* That's all, stop editing! Happy blogging. */</code> please add this code:" );
-		$msg .= "<br/><br/><code>$out</code>";
-
-		echo "<div class='update-nag'>$msg</div>";
-	}
-}
-add_action( 'network_admin_notices', 'secret_salt_warning' );
-
+/**
+ * Displays an admin notice to upgrade all sites after a core upgrade.
+ *
+ * @since 3.0.0
+ *
+ * @global int $wp_db_version The version number of the database.
+ */
 function site_admin_notice() {
 	global $wp_db_version;
 	if ( !is_super_admin() )
@@ -533,6 +729,18 @@ function site_admin_notice() {
 add_action( 'admin_notices', 'site_admin_notice' );
 add_action( 'network_admin_notices', 'site_admin_notice' );
 
+/**
+ * Avoids a collision between a site slug and a permalink slug.
+ *
+ * In a subdirectory install this will make sure that a site and a post do not use the
+ * same subdirectory by checking for a site with the same name as a new post.
+ *
+ * @since 3.0.0
+ *
+ * @param array $data    An array of post data.
+ * @param array $postarr An array of posts. Not currently used.
+ * @return array The new array of post data after checking for collisions.
+ */
 function avoid_blog_page_permalink_collision( $data, $postarr ) {
 	if ( is_subdomain_install() )
 		return $data;
@@ -556,6 +764,14 @@ function avoid_blog_page_permalink_collision( $data, $postarr ) {
 }
 add_filter( 'wp_insert_post_data', 'avoid_blog_page_permalink_collision', 10, 2 );
 
+/**
+ * Handles the display of choosing a user's primary site.
+ *
+ * This displays the user's primary site and allows the user to choose
+ * which site is primary.
+ *
+ * @since 3.0.0
+ */
 function choose_primary_blog() {
 	?>
 	<table class="form-table">
@@ -595,7 +811,11 @@ function choose_primary_blog() {
 	<?php if ( in_array( get_site_option( 'registration' ), array( 'all', 'blog' ) ) ) : ?>
 		<tr>
 			<th scope="row" colspan="2" class="th-full">
-				<a href="<?php echo apply_filters( 'wp_signup_location', network_site_url( 'wp-signup.php' ) ); ?>"><?php _e( 'Create a New Site' ); ?></a>
+				<?php
+				/** This filter is documented in wp-login.php */
+				$sign_up_url = apply_filters( 'wp_signup_location', network_site_url( 'wp-signup.php' ) );
+				?>
+				<a href="<?php echo esc_url( $sign_up_url ); ?>"><?php _e( 'Create a New Site' ); ?></a>
 			</th>
 		</tr>
 	<?php endif; ?>
@@ -604,18 +824,27 @@ function choose_primary_blog() {
 }
 
 /**
- * Grants super admin privileges.
+ * Grants Super Admin privileges.
  *
  * @since 3.0.0
- * @param int $user_id
+ *
+ * @param int $user_id ID of the user to be granted Super Admin privileges.
+ * @return bool True on success, false on failure. This can fail when the user is
+ *              already a super admin or when the `$super_admins` global is defined.
  */
 function grant_super_admin( $user_id ) {
-	global $super_admins;
-
 	// If global super_admins override is defined, there is nothing to do here.
-	if ( isset($super_admins) )
+	if ( isset( $GLOBALS['super_admins'] ) ) {
 		return false;
+	}
 
+	/**
+	 * Fires before the user is granted Super Admin privileges.
+	 *
+	 * @since 3.0.0
+	 *
+	 * @param int $user_id ID of the user that is about to be granted Super Admin privileges.
+	 */
 	do_action( 'grant_super_admin', $user_id );
 
 	// Directly fetch site_admins instead of using get_super_admins()
@@ -625,6 +854,14 @@ function grant_super_admin( $user_id ) {
 	if ( $user && ! in_array( $user->user_login, $super_admins ) ) {
 		$super_admins[] = $user->user_login;
 		update_site_option( 'site_admins' , $super_admins );
+
+		/**
+		 * Fires after the user is granted Super Admin privileges.
+		 *
+		 * @since 3.0.0
+		 *
+		 * @param int $user_id ID of the user that was granted Super Admin privileges.
+		 */
 		do_action( 'granted_super_admin', $user_id );
 		return true;
 	}
@@ -632,28 +869,45 @@ function grant_super_admin( $user_id ) {
 }
 
 /**
- * Revokes super admin privileges.
+ * Revokes Super Admin privileges.
  *
  * @since 3.0.0
- * @param int $user_id
+ *
+ * @param int $user_id ID of the user Super Admin privileges to be revoked from.
+ * @return bool True on success, false on failure. This can fail when the user's email
+ *              is the network admin email or when the `$super_admins` global is defined.
  */
 function revoke_super_admin( $user_id ) {
-	global $super_admins;
-
 	// If global super_admins override is defined, there is nothing to do here.
-	if ( isset($super_admins) )
+	if ( isset( $GLOBALS['super_admins'] ) ) {
 		return false;
+	}
 
+	/**
+	 * Fires before the user's Super Admin privileges are revoked.
+	 *
+	 * @since 3.0.0
+	 *
+	 * @param int $user_id ID of the user Super Admin privileges are being revoked from.
+	 */
 	do_action( 'revoke_super_admin', $user_id );
 
 	// Directly fetch site_admins instead of using get_super_admins()
 	$super_admins = get_site_option( 'site_admins', array( 'admin' ) );
 
 	$user = get_userdata( $user_id );
-	if ( $user && $user->user_email != get_site_option( 'admin_email' ) ) {
+	if ( $user && 0 !== strcasecmp( $user->user_email, get_site_option( 'admin_email' ) ) ) {
 		if ( false !== ( $key = array_search( $user->user_login, $super_admins ) ) ) {
 			unset( $super_admins[$key] );
 			update_site_option( 'site_admins', $super_admins );
+
+			/**
+			 * Fires after the user's Super Admin privileges are revoked.
+			 *
+			 * @since 3.0.0
+			 *
+			 * @param int $user_id ID of the user Super Admin privileges were revoked from.
+			 */
 			do_action( 'revoked_super_admin', $user_id );
 			return true;
 		}
@@ -662,21 +916,32 @@ function revoke_super_admin( $user_id ) {
 }
 
 /**
- * Whether or not we can edit this network from this page
+ * Whether or not we can edit this network from this page.
  *
- * By default editing of network is restricted to the Network Admin for that site_id this allows for this to be overridden
+ * By default editing of network is restricted to the Network Admin for that `$site_id`
+ * this allows for this to be overridden.
  *
  * @since 3.1.0
- * @param integer $site_id The network/site id to check.
+ *
+ * @param int $site_id The network/site ID to check.
+ * @return bool True if network can be edited, otherwise false.
  */
 function can_edit_network( $site_id ) {
 	global $wpdb;
 
-	if ($site_id == $wpdb->siteid )
+	if ( $site_id == $wpdb->siteid )
 		$result = true;
 	else
 		$result = false;
 
+	/**
+	 * Filter whether this network can be edited from this page.
+	 *
+	 * @since 3.1.0
+	 *
+	 * @param bool $result  Whether the network can be edited from this page.
+	 * @param int  $site_id The network/site ID to check.
+	 */
 	return apply_filters( 'can_edit_network', $result, $site_id );
 }
 
@@ -684,13 +949,14 @@ function can_edit_network( $site_id ) {
  * Thickbox image paths for Network Admin.
  *
  * @since 3.1.0
+ *
  * @access private
  */
 function _thickbox_path_admin_subfolder() {
 ?>
 <script type="text/javascript">
 //<![CDATA[
-var tb_pathToImage = "../../wp-includes/js/thickbox/loadingAnimation.gif";
+var tb_pathToImage = "<?php echo includes_url( 'js/thickbox/loadingAnimation.gif', 'relative' ); ?>";
 //]]>
 </script>
 <?php
