@@ -16,10 +16,7 @@
  * Everything will be stripped of tags and characters converted, when the values
  * are retrieved for use in the feeds.
  *
- * @package WordPress
- * @subpackage Feed
  * @since 1.5.1
- * @uses apply_filters() Calls 'get_bloginfo_rss' hook with two parameters.
  * @see get_bloginfo() For the list of possible values to display.
  *
  * @param string $show See get_bloginfo() for possible values.
@@ -27,7 +24,18 @@
  */
 function get_bloginfo_rss($show = '') {
 	$info = strip_tags(get_bloginfo($show));
-	return apply_filters('get_bloginfo_rss', convert_chars($info), $show);
+	/**
+	 * Filter the bloginfo for use in RSS feeds.
+	 *
+	 * @since 2.2.0
+	 *
+	 * @see convert_chars()
+	 * @see get_bloginfo()
+	 *
+	 * @param string $info Converted string value of the blog information.
+	 * @param string $show The type of blog information to retrieve.
+	 */
+	return apply_filters( 'get_bloginfo_rss', convert_chars( $info ), $show );
 }
 
 /**
@@ -37,16 +45,23 @@ function get_bloginfo_rss($show = '') {
  * Everything will be stripped of tags and characters converted, when the values
  * are retrieved for use in the feeds.
  *
- * @package WordPress
- * @subpackage Feed
  * @since 0.71
- * @uses apply_filters() Calls 'bloginfo_rss' hook with two parameters.
  * @see get_bloginfo() For the list of possible values to display.
  *
  * @param string $show See get_bloginfo() for possible values.
  */
 function bloginfo_rss($show = '') {
-	echo apply_filters('bloginfo_rss', get_bloginfo_rss($show), $show);
+	/**
+	 * Filter the bloginfo for display in RSS feeds.
+	 *
+	 * @since 2.1.0
+	 *
+	 * @see get_bloginfo()
+	 *
+	 * @param string $rss_container RSS container for the blog information.
+	 * @param string $show          The type of blog information to retrieve.
+	 */
+	echo apply_filters( 'bloginfo_rss', get_bloginfo_rss( $show ), $show );
 }
 
 /**
@@ -55,34 +70,50 @@ function bloginfo_rss($show = '') {
  * The default feed is 'rss2', unless a plugin changes it through the
  * 'default_feed' filter.
  *
- * @package WordPress
- * @subpackage Feed
- * @since 2.5
- * @uses apply_filters() Calls 'default_feed' hook on the default feed string.
+ * @since 2.5.0
  *
  * @return string Default feed, or for example 'rss2', 'atom', etc.
  */
 function get_default_feed() {
-	$default_feed = apply_filters('default_feed', 'rss2');
+	/**
+	 * Filter the default feed type.
+	 *
+	 * @since 2.5.0
+	 *
+	 * @param string $feed_type Type of default feed. Possible values include 'rss2', 'atom'.
+	 *                          Default 'rss2'.
+	 */
+	$default_feed = apply_filters( 'default_feed', 'rss2' );
 	return 'rss' == $default_feed ? 'rss2' : $default_feed;
 }
 
 /**
  * Retrieve the blog title for the feed title.
  *
- * @package WordPress
- * @subpackage Feed
  * @since 2.2.0
- * @uses apply_filters() Calls 'get_wp_title_rss' hook on title.
- * @uses wp_title() See function for $sep parameter usage.
  *
- * @param string $sep Optional.How to separate the title. See wp_title() for more info.
+ * @param string $sep Optional. How to separate the title. See wp_title() for more info.
  * @return string Error message on failure or blog title on success.
  */
-function get_wp_title_rss($sep = '&#187;') {
-	$title = wp_title($sep, false);
-	if ( is_wp_error( $title ) )
+function get_wp_title_rss( $sep = '&#187;' ) {
+	$title = wp_title( $sep, false );
+
+	if ( is_wp_error( $title ) ) {
 		return $title->get_error_message();
+	}
+
+	if ( $title && $sep && ' ' !== substr( $title, 0, 1 ) ) {
+		$title = " $sep " . $title;
+	}
+
+	/**
+	 * Filter the blog title for use as the feed title.
+	 *
+	 * @since 2.2.0
+	 *
+	 * @param string $title The current blog title.
+	 * @param string $sep   Separator used by wp_title().
+	 */
 	$title = apply_filters( 'get_wp_title_rss', $title, $sep );
 	return $title;
 }
@@ -90,41 +121,49 @@ function get_wp_title_rss($sep = '&#187;') {
 /**
  * Display the blog title for display of the feed title.
  *
- * @package WordPress
- * @subpackage Feed
  * @since 2.2.0
- * @uses apply_filters() Calls 'wp_title_rss' on the blog title.
  * @see wp_title() $sep parameter usage.
  *
  * @param string $sep Optional.
  */
 function wp_title_rss( $sep = '&#187;' ) {
+	/**
+	 * Filter the blog title for display of the feed title.
+	 *
+	 * @since 2.2.0
+	 *
+	 * @see get_wp_title_rss()
+	 *
+	 * @param string $wp_title The current blog title.
+	 * @param string $sep      Separator used by wp_title().
+	 */
 	echo apply_filters( 'wp_title_rss', get_wp_title_rss( $sep ), $sep );
 }
 
 /**
  * Retrieve the current post title for the feed.
  *
- * @package WordPress
- * @subpackage Feed
  * @since 2.0.0
- * @uses apply_filters() Calls 'the_title_rss' on the post title.
  *
  * @return string Current post title.
  */
 function get_the_title_rss() {
 	$title = get_the_title();
-	$title = apply_filters('the_title_rss', $title);
+	/**
+	 * Filter the post title for use in a feed.
+	 *
+	 * @since 1.2.0
+	 *
+	 * @param string $title The current post title.
+	 */
+	$title = apply_filters( 'the_title_rss', $title );
 	return $title;
 }
 
 /**
  * Display the post title in the feed.
  *
- * @package WordPress
- * @subpackage Feed
  * @since 0.71
- * @uses get_the_title_rss() Used to retrieve current post title.
  */
 function the_title_rss() {
 	echo get_the_title_rss();
@@ -133,10 +172,7 @@ function the_title_rss() {
 /**
  * Retrieve the post content for feeds.
  *
- * @package WordPress
- * @subpackage Feed
  * @since 2.9.0
- * @uses apply_filters() Calls 'the_content_feed' on the content before processing.
  * @see get_the_content()
  *
  * @param string $feed_type The type of feed. rss2 | atom | rss | rdf
@@ -146,19 +182,25 @@ function get_the_content_feed($feed_type = null) {
 	if ( !$feed_type )
 		$feed_type = get_default_feed();
 
-	$content = apply_filters('the_content', get_the_content());
+	/** This filter is documented in wp-includes/post-template.php */
+	$content = apply_filters( 'the_content', get_the_content() );
 	$content = str_replace(']]>', ']]&gt;', $content);
-	return apply_filters('the_content_feed', $content, $feed_type);
+	/**
+	 * Filter the post content for use in feeds.
+	 *
+	 * @since 2.9.0
+	 *
+	 * @param string $content   The current post content.
+	 * @param string $feed_type Type of feed. Possible values include 'rss2', 'atom'.
+	 *                          Default 'rss2'.
+	 */
+	return apply_filters( 'the_content_feed', $content, $feed_type );
 }
 
 /**
  * Display the post content for feeds.
  *
- * @package WordPress
- * @subpackage Feed
  * @since 2.9.0
- * @uses apply_filters() Calls 'the_content_feed' on the content before processing.
- * @see get_the_content()
  *
  * @param string $feed_type The type of feed. rss2 | atom | rss | rdf
  */
@@ -169,26 +211,34 @@ function the_content_feed($feed_type = null) {
 /**
  * Display the post excerpt for the feed.
  *
- * @package WordPress
- * @subpackage Feed
  * @since 0.71
- * @uses apply_filters() Calls 'the_excerpt_rss' hook on the excerpt.
  */
 function the_excerpt_rss() {
 	$output = get_the_excerpt();
-	echo apply_filters('the_excerpt_rss', $output);
+	/**
+	 * Filter the post excerpt for a feed.
+	 *
+	 * @since 1.2.0
+	 *
+	 * @param string $output The current post excerpt.
+	 */
+	echo apply_filters( 'the_excerpt_rss', $output );
 }
 
 /**
  * Display the permalink to the post for use in feeds.
  *
- * @package WordPress
- * @subpackage Feed
  * @since 2.3.0
- * @uses apply_filters() Call 'the_permalink_rss' on the post permalink
  */
 function the_permalink_rss() {
-	echo esc_url( apply_filters('the_permalink_rss', get_permalink() ));
+	/**
+	 * Filter the permalink to the post for use in feeds.
+	 *
+	 * @since 2.3.0
+	 *
+	 * @param string $post_permalink The current post permalink.
+	 */
+	echo esc_url( apply_filters( 'the_permalink_rss', get_permalink() ) );
 }
 
 /**
@@ -198,14 +248,20 @@ function the_permalink_rss() {
  * @return none
  */
 function comments_link_feed() {
+	/**
+	 * Filter the comments permalink for the current post.
+	 *
+	 * @since 3.6.0
+	 *
+	 * @param string $comment_permalink The current comment permalink with
+	 *                                  '#comments' appended.
+	 */
 	echo esc_url( apply_filters( 'comments_link_feed', get_comments_link() ) );
 }
 
 /**
  * Display the feed GUID for the current comment.
  *
- * @package WordPress
- * @subpackage Feed
  * @since 2.5.0
  *
  * @param int|object $comment_id Optional comment object or id. Defaults to global comment object.
@@ -217,12 +273,10 @@ function comment_guid($comment_id = null) {
 /**
  * Retrieve the feed GUID for the current comment.
  *
- * @package WordPress
- * @subpackage Feed
  * @since 2.5.0
  *
  * @param int|object $comment_id Optional comment object or id. Defaults to global comment object.
- * @return bool|string false on failure or guid for comment on success.
+ * @return false|string false on failure or guid for comment on success.
  */
 function get_comment_guid($comment_id = null) {
 	$comment = get_comment($comment_id);
@@ -239,29 +293,41 @@ function get_comment_guid($comment_id = null) {
  * @since 1.5.0
  */
 function comment_link() {
+	/**
+	 * Filter the current comment's permalink.
+	 *
+	 * @since 3.6.0
+	 *
+	 * @see get_comment_link()
+	 *
+	 * @param string $comment_permalink The current comment permalink.
+	 */
 	echo esc_url( apply_filters( 'comment_link', get_comment_link() ) );
 }
 
 /**
  * Retrieve the current comment author for use in the feeds.
  *
- * @package WordPress
- * @subpackage Feed
  * @since 2.0.0
- * @uses apply_filters() Calls 'comment_author_rss' hook on comment author.
- * @uses get_comment_author()
  *
  * @return string Comment Author
  */
 function get_comment_author_rss() {
-	return apply_filters('comment_author_rss', get_comment_author() );
+	/**
+	 * Filter the current comment author for use in a feed.
+	 *
+	 * @since 1.5.0
+	 *
+	 * @see get_comment_author()
+	 *
+	 * @param string $comment_author The current comment author.
+	 */
+	return apply_filters( 'comment_author_rss', get_comment_author() );
 }
 
 /**
  * Display the current comment author in the feed.
  *
- * @package WordPress
- * @subpackage Feed
  * @since 1.0.0
  */
 function comment_author_rss() {
@@ -271,15 +337,18 @@ function comment_author_rss() {
 /**
  * Display the current comment content for use in the feeds.
  *
- * @package WordPress
- * @subpackage Feed
  * @since 1.0.0
- * @uses apply_filters() Calls 'comment_text_rss' filter on comment content.
- * @uses get_comment_text()
  */
 function comment_text_rss() {
 	$comment_text = get_comment_text();
-	$comment_text = apply_filters('comment_text_rss', $comment_text);
+	/**
+	 * Filter the current comment content for use in a feed.
+	 *
+	 * @since 1.5.0
+	 *
+	 * @param string $comment_text The content of the current comment.
+	 */
+	$comment_text = apply_filters( 'comment_text_rss', $comment_text );
 	echo $comment_text;
 }
 
@@ -290,10 +359,7 @@ function comment_text_rss() {
  * retrieved and have feed markup added, so that they can easily be added to the
  * RSS2, Atom, or RSS1 and RSS0.91 RDF feeds.
  *
- * @package WordPress
- * @subpackage Feed
  * @since 2.1.0
- * @uses apply_filters()
  *
  * @param string $type Optional, default is the type returned by get_default_feed().
  * @return string All of the post categories for displaying in the feed.
@@ -324,19 +390,26 @@ function get_the_category_rss($type = null) {
 		if ( 'rdf' == $type )
 			$the_list .= "\t\t<dc:subject><![CDATA[$cat_name]]></dc:subject>\n";
 		elseif ( 'atom' == $type )
-			$the_list .= sprintf( '<category scheme="%1$s" term="%2$s" />', esc_attr( apply_filters( 'get_bloginfo_rss', get_bloginfo( 'url' ) ) ), esc_attr( $cat_name ) );
+			$the_list .= sprintf( '<category scheme="%1$s" term="%2$s" />', esc_attr( get_bloginfo_rss( 'url' ) ), esc_attr( $cat_name ) );
 		else
 			$the_list .= "\t\t<category><![CDATA[" . @html_entity_decode( $cat_name, ENT_COMPAT, get_option('blog_charset') ) . "]]></category>\n";
 	}
 
-	return apply_filters('the_category_rss', $the_list, $type);
+	/**
+	 * Filter all of the post categories for display in a feed.
+	 *
+	 * @since 1.2.0
+	 *
+	 * @param string $the_list All of the RSS post categories.
+	 * @param string $type     Type of feed. Possible values include 'rss2', 'atom'.
+	 *                         Default 'rss2'.
+	 */
+	return apply_filters( 'the_category_rss', $the_list, $type );
 }
 
 /**
  * Display the post categories in the feed.
  *
- * @package WordPress
- * @subpackage Feed
  * @since 0.71
  * @see get_the_category_rss() For better explanation.
  *
@@ -351,8 +424,6 @@ function the_category_rss($type = null) {
  *
  * The two possible values are either 'xhtml' or 'html'.
  *
- * @package WordPress
- * @subpackage Feed
  * @since 2.2.0
  */
 function html_type_rss() {
@@ -376,11 +447,7 @@ function html_type_rss() {
  * enclosure(s) consist of enclosure HTML tag(s) with a URI and other
  * attributes.
  *
- * @package WordPress
- * @subpackage Template
  * @since 1.5.0
- * @uses apply_filters() Calls 'rss_enclosure' hook on rss enclosure.
- * @uses get_post_custom() To get the current post enclosure metadata.
  */
 function rss_enclosure() {
 	if ( post_password_required() )
@@ -395,7 +462,14 @@ function rss_enclosure() {
 				$t = preg_split('/[ \t]/', trim($enclosure[2]) );
 				$type = $t[0];
 
-				echo apply_filters('rss_enclosure', '<enclosure url="' . trim(htmlspecialchars($enclosure[0])) . '" length="' . trim($enclosure[1]) . '" type="' . $type . '" />' . "\n");
+				/**
+				 * Filter the RSS enclosure HTML link tag for the current post.
+				 *
+				 * @since 2.2.0
+				 *
+				 * @param string $html_link_tag The HTML link tag with a URI and other attributes.
+				 */
+				echo apply_filters( 'rss_enclosure', '<enclosure url="' . esc_url( trim( $enclosure[0] ) ) . '" length="' . absint( trim( $enclosure[1] ) ) . '" type="' . esc_attr( $type ) . '" />' . "\n" );
 			}
 		}
 	}
@@ -412,11 +486,7 @@ function rss_enclosure() {
  * metadata field and parses the value to display the enclosure(s). The
  * enclosure(s) consist of link HTML tag(s) with a URI and other attributes.
  *
- * @package WordPress
- * @subpackage Template
  * @since 2.2.0
- * @uses apply_filters() Calls 'atom_enclosure' hook on atom enclosure.
- * @uses get_post_custom() To get the current post enclosure metadata.
  */
 function atom_enclosure() {
 	if ( post_password_required() )
@@ -426,7 +496,14 @@ function atom_enclosure() {
 		if ($key == 'enclosure') {
 			foreach ( (array) $val as $enc ) {
 				$enclosure = explode("\n", $enc);
-				echo apply_filters('atom_enclosure', '<link href="' . trim(htmlspecialchars($enclosure[0])) . '" rel="enclosure" length="' . trim($enclosure[1]) . '" type="' . trim($enclosure[2]) . '" />' . "\n");
+				/**
+				 * Filter the atom enclosure HTML link tag for the current post.
+				 *
+				 * @since 2.2.0
+				 *
+				 * @param string $html_link_tag The HTML link tag with a URI and other attributes.
+				 */
+				echo apply_filters( 'atom_enclosure', '<link href="' . esc_url( trim( $enclosure[0] ) ) . '" rel="enclosure" length="' . absint( trim( $enclosure[1] ) ) . '" type="' . esc_attr( trim( $enclosure[2] ) ) . '" />' . "\n" );
 			}
 		}
 	}
@@ -444,9 +521,7 @@ function atom_enclosure() {
  *
  * @link http://www.atomenabled.org/developers/syndication/atom-format-spec.php#rfc.section.3.1
  *
- * @package WordPress
- * @subpackage Feed
- * @since 2.5
+ * @since 2.5.0
  *
  * @param string $data Input string
  * @return array array(type, value)
@@ -482,20 +557,26 @@ function prep_atom_text_construct($data) {
  *
  * Generate a correct link for the atom:self element.
  *
- * @package WordPress
- * @subpackage Feed
- * @since 2.5
+ * @since 2.5.0
  */
 function self_link() {
 	$host = @parse_url(home_url());
+	/**
+	 * Filter the current feed URL.
+	 *
+	 * @since 3.6.0
+	 *
+	 * @see set_url_scheme()
+	 * @see wp_unslash()
+	 *
+	 * @param string $feed_link The link for the feed with set URL scheme.
+	 */
 	echo esc_url( apply_filters( 'self_link', set_url_scheme( 'http://' . $host['host'] . wp_unslash( $_SERVER['REQUEST_URI'] ) ) ) );
 }
 
 /**
  * Return the content type for specified feed type.
  *
- * @package WordPress
- * @subpackage Feed
  * @since 2.8.0
  */
 function feed_content_type( $type = '' ) {
@@ -512,13 +593,22 @@ function feed_content_type( $type = '' ) {
 
 	$content_type = ( !empty($types[$type]) ) ? $types[$type] : 'application/octet-stream';
 
+	/**
+	 * Filter the content type for a specific feed type.
+	 *
+	 * @since 2.8.0
+	 *
+	 * @param string $content_type Content type indicating the type of data that a feed contains.
+	 * @param string $type         Type of feed. Possible values include 'rss2', 'atom'.
+	 *                             Default 'rss2'.
+	 */
 	return apply_filters( 'feed_content_type', $content_type, $type );
 }
 
 /**
  * Build SimplePie object based on RSS or Atom feed from URL.
  *
- * @since 2.8
+ * @since 2.8.0
  *
  * @param mixed $url URL of feed to retrieve. If an array of URLs, the feeds are merged
  * using SimplePie's multifeed feature.
@@ -540,7 +630,16 @@ function fetch_feed( $url ) {
 	$feed->set_file_class( 'WP_SimplePie_File' );
 
 	$feed->set_feed_url( $url );
+	/** This filter is documented in wp-includes/class-feed.php */
 	$feed->set_cache_duration( apply_filters( 'wp_feed_cache_transient_lifetime', 12 * HOUR_IN_SECONDS, $url ) );
+	/**
+	 * Fires just before processing the SimplePie feed object.
+	 *
+	 * @since 3.0.0
+	 *
+	 * @param object &$feed SimplePie feed object, passed by reference.
+	 * @param mixed  $url   URL of feed to retrieve. If an array of URLs, the feeds are merged.
+	 */
 	do_action_ref_array( 'wp_feed_options', array( &$feed, $url ) );
 	$feed->init();
 	$feed->handle_content_type();
